 /* 
  * UAE - The Un*x Amiga Emulator
  * 
  * Support for Linux/USS sound
  * 
  * Copyright 1997 Bernd Schmidt
  */

#include "sysconfig.h"
#include "sysdeps.h"

#include "config.h"
#include "options.h"
#include "memory.h"
#include "custom.h"
#include "gensound.h"
#include "sd-sound.h"

#include <sys/ioctl.h>

#ifdef HAVE_SYS_SOUNDCARD_H
#include <sys/soundcard.h>
#elif defined HAVE_MACHINE_SOUNDCARD_H
#include <machine/soundcard.h>
#elif defined HAVE_SOUNDCARD_H
#include <soundcard.h>
#else
#error "Something went wrong during configuration."
#endif

#ifdef __FreeBSD__
#ifndef AFMT_S16_NE
#include <machine/endian.h>
#if BYTE_ORDER == LITTLE_ENDIAN
#   define AFMT_S16_NE AFMT_S16_LE              /* native endian signed 16 */
#else
#   define AFMT_S16_NE AFMT_S16_BE
#endif
#endif
#endif

#include "uade.h"

int sound_fd;
static int have_sound = 0;
static unsigned long formats;

uae_u16 sndbuffer[44100];
uae_u16 *sndbufpt;
int sndbufsize;

static int exact_log2 (int v)
{
    int l = 0;
    while ((v >>= 1) != 0)
	l++;
    return l;
}

void close_sound (void)
{
    if (have_sound)
	close (sound_fd);
}

/* Try to determine whether sound is available.  This is only for GUI purposes.  */
int setup_sound (void)
{
  if (uade_local_sound) {
    sound_available = 0;
    sound_fd = open ("/dev/dsp", O_WRONLY);
    if (sound_fd < 0) {
      perror ("uade: can't open /dev/dsp");
      if (errno == EBUSY) {
	/* We can hope, can't we ;) */
	sound_available = 1;
	return 1;
      }
      return 0;
    }
    if (ioctl (sound_fd, SNDCTL_DSP_GETFMTS, &formats) == -1) {
      perror ("uade: ioctl failed - can't use sound");
      close (sound_fd);
      return 0;
    }
    close(sound_fd);
  }
  sound_available = 1;
  return 1;
}

int init_sound (void)
{
  int tmp;
  int rate;
  int dspbits;

  if (currprefs.sound_maxbsiz < 128 || currprefs.sound_maxbsiz > 16384) {
    fprintf (stderr, "Sound buffer size %d out of range.\n", currprefs.sound_maxbsiz);
    currprefs.sound_maxbsiz = 8192;
  }

  dspbits = currprefs.sound_bits;
  rate    = currprefs.sound_freq;

  if (uade_local_sound) {
    sound_fd = open ("/dev/dsp", O_WRONLY);
    have_sound = !(sound_fd < 0);
    if (!have_sound) {
      perror ("uade: can't open /dev/dsp");
      if (errno != EBUSY)
	sound_available = 0;
      return 0;
    }

    if (ioctl (sound_fd, SNDCTL_DSP_GETFMTS, &formats) == -1) {
      perror ("ioctl failed - can't use sound");
      close (sound_fd);
      have_sound = 0;
      return 0;
    }

    /* max fragments = 4, fragment size = log2(maxbsiz)  */
    tmp = 0x00040000 + exact_log2 (currprefs.sound_maxbsiz);
    ioctl (sound_fd, SNDCTL_DSP_SETFRAGMENT, &tmp);
    ioctl (sound_fd, SNDCTL_DSP_GETBLKSIZE, &sndbufsize);

    if (dspbits == 16) {
      tmp = AFMT_S16_NE;
      if (!(formats & tmp)) {
	fprintf(stderr, "uade: init_sound error: can not use AFMT_S16_NE\n");
	return 0;
      }
    } else {
      tmp = AFMT_U8;
      if (!(formats & tmp)) {
	fprintf(stderr, "uade: init_sound error: can not use AFMT_U8\n");
	return 0;
      }
    }
    if (ioctl (sound_fd, SNDCTL_DSP_SETFMT, &tmp) == -1) {
      fprintf(stderr, "uade: can't init sound with %d bits", dspbits);
      return 0;
    }

    tmp = currprefs.stereo;
    ioctl (sound_fd, SNDCTL_DSP_STEREO, &tmp);
    ioctl (sound_fd, SNDCTL_DSP_SPEED, &rate);

    ioctl (sound_fd, SOUND_PCM_READ_RATE, &rate);
    /* Some soundcards have a bit of tolerance here. */
    if (rate < currprefs.sound_freq * 90 / 100 || rate > currprefs.sound_freq * 110 / 100) {
      fprintf (stderr, "Can't use sound with desired frequency %d\n", currprefs.sound_freq);
      return 0;
    }
  } else {
    have_sound = 1;
    formats = -1;
    sndbufsize = 4096;
  }

  sample_evtime = (long) maxhpos * maxvpos * 50 / rate;
  if (dspbits == 16) {
    init_sound_table16 ();
    sample_handler = currprefs.stereo ? sample16s_handler : sample16_handler;
  } else {
    init_sound_table8 ();
    sample_handler = currprefs.stereo ? sample8s_handler : sample8_handler;
  }
  sound_available = 1;

  /*  if (uade_local_sound) {
    fprintf (stderr,"Sound driver found and configured for %d bits %s at %d Hz, buffer is %d bytes\n", dspbits, currprefs.stereo ? "stereo" : "mono", rate, sndbufsize);
  }
  */

  sndbufpt = sndbuffer;
#ifdef FRAME_RATE_HACK
  vsynctime = vsynctime * 9 / 10;
#endif	
  return 1;
}

/* this should be called between subsongs when remote slave changes subsong */
void flush_sound (void)
{
  sndbufpt = sndbuffer;
}
