/* Copyright 2003 Heikki Orsila <heikki.orsila@ee.tut.fi>

TODO:
interactive mode: ls, command launch support, pwd, ...
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <glob.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <limits.h>
#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

#include "../config.h"

#ifdef HAVE_SHELL_INTERACTION
#include "unix-shell-int.h"
#endif
#include "uade-os.h"
#include "../osdep/strl.c"
#include "unix-shell.h"

#ifndef HAVE_MACOSX /* macosx doesn't have ftw. hell.. */
#include <ftw.h>
#endif

struct songnode {
  struct songnode *next;
  char filename[PATH_MAX];
};

static struct songnode *SongList = 0;

static void (*us_interrupt)(void) = 0;

/* catch CTRL-C and cause REBOOT */
static void us_sigint_handler(int signum) {
  signum = signum;
  if (us_interrupt) {
    us_interrupt();
  } else {
    fprintf(stderr, "uade: non-initialized sigint interrupt\n");
  }
}

static void us_setup(void (*interrupt)(void)) {
  us_interrupt = interrupt;
  /* create CTRL-C break for cmdline tool */
  if (!uade_create_signalhandler(us_sigint_handler, UADE_SIGINT)) {
    fprintf(stderr, "uade: failed to setup ctrl-c handler. it won't work.\n");
  }
}

#ifndef HAVE_MACOSX
static int ftw_func(const char *file, const struct stat *sb, int flag, void *arg) {
  if (flag == UADE_FTW_F) {
    char name[PATH_MAX];
    strlcpy(name, file, sizeof(name));
    printf("enqueuing: %s\n", name);
    us_enqueue_song(name);
  }
  return 0;
}
#endif

void us_enqueue_song(char *file) {
  struct songnode *s = SongList;
  struct songnode *new;

  /* allow queuing of zero length string */
  if (uade_is_regular_file(file) || strlen(file) == 0) {
    new = malloc(sizeof(struct songnode));
    new->next = 0;
    strlcpy(new->filename, file, sizeof(new->filename));
    if (s) {
      while(s->next) s = s->next;
      s->next = new;
    } else {
      SongList = new;
    }
    return;
  }

#ifndef HAVE_MACOSX
  do {
    struct stat st;
    if (!stat(file, &st)) {
      if (S_ISDIR(st.st_mode)) {
	uade_ftw(file, ftw_func, 20, 0);
      }
    }
  } while (0);
#endif
}

int us_song_list_empty(void) {
  return SongList ? 0 : 1;
}

void us_get_next(char *dst, int maxsize) {
  char playername[PATH_MAX];
  char tempname[PATH_MAX];
  struct songnode *t;
  if (!SongList)
    return;
  strlcpy(dst, SongList->filename, maxsize);
  t = SongList;
  SongList = SongList->next;
  free(t);
}

void us_functions(struct uade_local *local) {
  local->setup = us_setup;
  local->enqueue_song = us_enqueue_song;
  local->get_next = us_get_next;
#ifdef HAVE_SHELL_INTERACTION
  local->interaction = us_interaction;
#endif
  local->song_list_empty = us_song_list_empty;
};
